﻿''' <summary>
''' Author: Oliver Schirm
''' Date: 16.10.2015
''' Description: This class removes white border from pictures.
''' </summary>
Public Class ImageManipulation
    ''' <summary>
    ''' Cuts the white borders and scales to original size.
    ''' </summary>
    Public Shared Function removeWhiteBorder(ByVal bmp As Bitmap) As Bitmap
        Dim xCropLeft As Integer = -1
        Dim xCropRight As Integer = -1
        Dim yCropTop As Integer = -1
        Dim yCropBottom As Integer = -1
        Dim bEnd As Boolean = False
        For x = 0 To bmp.Width - 1
            For y = 0 To bmp.Height - 1
                If bmp.GetPixel(x, y).GetBrightness = 0 Then
                    If xCropLeft = -1 Then
                        xCropLeft = x
                        bEnd = True
                        Exit For
                    End If
                End If
            Next
            If bEnd = True Then
                Exit For
            End If
        Next
        bEnd = False
        For x = bmp.Width - 1 To 0 Step -1
            For y = 0 To bmp.Height - 1
                If bmp.GetPixel(x, y).GetBrightness = 0 Then
                    If xCropRight = -1 Then
                        xCropRight = x
                        bEnd = True
                        Exit For
                    End If
                End If
            Next
            If bEnd = True Then
                Exit For
            End If
        Next
        bEnd = False
        For y = 0 To bmp.Height - 1
            For x = 0 To bmp.Width - 1
                If bmp.GetPixel(x, y).GetBrightness = 0 Then
                    If yCropTop = -1 Then
                        yCropTop = y
                        bEnd = True
                        Exit For
                    End If
                End If
            Next
            If bEnd = True Then
                Exit For
            End If
        Next
        bEnd = False
        For y = bmp.Height - 1 To 0 Step -1
            For x = 0 To bmp.Width - 1
                If bmp.GetPixel(x, y).GetBrightness = 0 Then
                    If yCropBottom = -1 Then
                        yCropBottom = y
                        bEnd = True
                        Exit For
                    End If
                End If
            Next
            If bEnd = True Then
                Exit For
            End If
        Next
        Dim bmpCropped As Bitmap = crop(bmp, New Rectangle(xCropLeft, yCropTop, xCropRight - xCropLeft, yCropBottom - yCropTop))
        Dim bmpScaled As Bitmap = scale(bmpCropped, bmp.Width, bmp.Height)
        Return bmpScaled
    End Function
    ''' <summary>
    ''' Crops the given bitmap.
    ''' </summary>
    Private Shared Function crop(ByVal bmp As Bitmap, ByVal rectCrop As Rectangle) As Bitmap
        Dim bmpCrop = New Bitmap(rectCrop.Width, rectCrop.Height)
        Using grp = Graphics.FromImage(bmpCrop)
            grp.DrawImage(bmp, New Rectangle(0, 0, rectCrop.Width, rectCrop.Height), rectCrop, GraphicsUnit.Pixel)
        End Using
        Return bmpCrop
    End Function
    ''' <summary>
    ''' Scales bitmap to given size.
    ''' </summary>
    Public Shared Function scale(ByVal bmp As Bitmap, ByVal iWidth As Integer, ByVal iHeight As Integer) As Bitmap
        Dim bmpScaled As New Bitmap(iWidth, iHeight)
        Dim g As Graphics = Graphics.FromImage(bmpScaled)
        g.DrawImage(bmp, 0, 0, iWidth, iHeight)
        g.Dispose()
        Return bmpScaled
    End Function
    ''' <summary>
    ''' Creates a new Image containing the same image only rotated.
    ''' </summary>
    ''' <param name="image">The <see cref="System.Drawing.Image"/> to rotate</param>
    ''' <param name="angle">The amount to rotate the image, clockwise, in degrees</param>
    ''' <returns>A new <see cref="System.Drawing.Bitmap"/> of the same size rotated.</returns>
    ''' <exception cref="System.ArgumentNullException">Thrown if <see cref="image"/> is null.</exception>
    Public Shared Function rotateImage(image As Image, angle As Single) As Bitmap
        Return RotateImage(image, New PointF(CSng(image.Width) / 2, CSng(image.Height) / 2), angle)
    End Function
    ''' <summary>
    ''' Creates a new Image containing the same image only rotated.
    ''' </summary>
    ''' <param name="image">The <see cref="System.Drawing.Image"/> to rotate</param>
    ''' <param name="offset">The position to rotate from.</param>
    ''' <param name="angle">The amount to rotate the image, clockwise, in degrees</param>
    ''' <returns>A new <see cref="System.Drawing.Bitmap"/> of the same size rotated.</returns>
    ''' <exception cref="System.ArgumentNullException">Thrown if <see cref="image"/> is null.</exception>
    Private Shared Function RotateImage(image As Image, offset As PointF, angle As Single) As Bitmap
        If image Is Nothing Then
            Throw New ArgumentNullException("image")
        End If
        'create a new empty bitmap to hold rotated image
        Dim rotatedBmp As New Bitmap(image.Width, image.Height)
        rotatedBmp.SetResolution(image.HorizontalResolution, image.VerticalResolution)
        'make a graphics object from the empty bitmap
        Dim g As Graphics = Graphics.FromImage(rotatedBmp)
        'Put the rotation point in the center of the image
        g.TranslateTransform(offset.X, offset.Y)
        'rotate the image
        g.RotateTransform(angle)
        'move the image back
        g.TranslateTransform(-offset.X, -offset.Y)
        'draw passed in image onto graphics object
        g.DrawImage(image, New PointF(0, 0))
        Return rotatedBmp
    End Function
End Class
